import requests
from datetime import datetime
from utils import format_phone_number, generate_access_token, generate_stk_password

class DarajaClient:
    def __init__(self, auth_url: str, consumer_key: str, consumer_secret: str, pass_key: str, shortcode: str, phone_number: str, call_back_url: str):
        """
        Initialize the MPESA Daraja 2.0 API client with required credentials and phone number.

        Args:
            url (str): The API base URL.
            consumer_key (str): Your consumer key for the API.
            consumer_secret (str): Your consumer secret for the API.
            pass_key (str): Security pass key for transactions.
            shortcode (str): Shortcode for transactions.
            phone_number (str): The phone number to format and validate.

        Raises:
            ValueError: If any required parameter is missing or invalid.
        """

        # Validate required parameters
        if not auth_url: raise ValueError("url cannot be empty!")
        if not consumer_key: raise ValueError("consumer_key cannot be empty!")
        if not consumer_secret: raise ValueError("consumer_secret cannot be empty!")
        if not pass_key: raise ValueError("pass_key cannot be empty!")
        if not shortcode: raise ValueError("shortcode cannot be empty!")
        if not phone_number: raise ValueError("phone_number cannot be empty!")
        if not call_back_url: raise ValueError("call_back_url cannot be empty!")


        # Initialize instance attributes
        self.auth_url = auth_url
        self.__consumer_key = consumer_key
        self.__consumer_secret = consumer_secret
        self.__pass_key = pass_key
        self.shortcode = shortcode
        self.phone_number = format_phone_number(phone_number)
        self.call_back_url = call_back_url

    @property
    def consumer_key(self):
        """Get the consumer key (read-only)."""
        return self.__consumer_key

    @property
    def consumer_secret(self):
        """Get the consumer secret (read-only)."""
        return self.__consumer_secret

    @property
    def pass_key(self):
        """Get the pass key (read-only)."""
        return self.__pass_key
    

    def send_stk_push(self, stk_push_url: str):
        # Get the access token using the utility function
        token = generate_access_token(self.consumer_key, self.consumer_secret, self.auth_url)
        
        # Generate the STK password using the utility function
        stk_password = generate_stk_password(self.shortcode, self.pass_key)
        
        timestamp = datetime.now().strftime('%Y%m%d%H%M%S')

        headers = {
            'Authorization': f'Bearer {token}',
            'Content-Type': 'application/json'
        }

        requestBody = {
            "BusinessShortCode": self.shortcode,
            "Password": stk_password,
            "Timestamp": timestamp,
            "TransactionType": "CustomerPayBillOnline", # "CustomerBuyGoodsOnline"
            "Amount": "1",
            "PartyA": self.phone_number,
            "PartyB": self.shortcode,
            "PhoneNumber": self.phone_number,
            "CallBackURL": self.call_back_url,
            "AccountReference": self.shortcode,
            "TransactionDesc": "MPESA ONLINE PAYMENT"
        }

        try:
            response = requests.post(stk_push_url, json=requestBody, headers=headers).json()
            response_code = response.get('ResponseCode')

            # Success
            if response_code == '0':
                return {
                    "message" : "success",
                    "stkpushID" : response.get("CheckoutRequestID"),
                    "info" : 'You can use and store this stkpushID in your db model to be used for payment confirm during callback from safaricom'
                }
            # failed
            if response_code != '0':
                return {
                    "message" : "failed",
                    "error" : response.get("errorMessage")
                }
            
            return {"message" : "request failed"}

        except Exception as e:
            print('Error:', str(e))



from core import DarajaClient
from decouple import config

url= "https://sandbox.safaricom.co.ke/oauth/v1/generate?grant_type=client_credentials"
stk_push_url ='https://sandbox.safaricom.co.ke/mpesa/stkpush/v1/processrequest'
call_back_url = 'https://amdeveloper-dpbzakb7gmg0ftgg.canadacentral-01.azurewebsites.net'
phone_number = '++254769507699'

      
cl =  DarajaClient(
            auth_url=url, 
            consumer_key= config('DARAJA_API_CONSUMER_KEY') , 
            consumer_secret=config('DARAJA_API_CONSUMER_SECRET'), 
            pass_key=config('DARAJA_API_PASS_KEY'),
            shortcode=config('DARAJA_API_SHORT_CODE'),
            phone_number=phone_number,
            call_back_url=call_back_url
)


if __name__ == '__main__':
    response = cl.send_stk_push(stk_push_url=stk_push_url)
    print(response)



import base64
import requests
from datetime import datetime


def format_phone_number(phone_number: str) -> str:
    """
    Formats a phone number by removing the '+' prefix if present and validating the result.

    Args:
        phone_number (str): The phone number to format.

    Returns:
        str: The formatted phone number without the '+' prefix.

    Raises:
        ValueError: If the phone number contains invalid characters.
    """
    
    
    # Remove '+' prefix if present
    formatted_phone = phone_number.lstrip('+')

    # Check if the formatted phone number contains only digits
    if not formatted_phone.isdigit():
        raise ValueError("Invalid phone number format. Phone number must contain only digits.")
    
    return formatted_phone



def generate_access_token(consumer_key: str, consumer_secret: str, url: str) -> str:
    try:
        # Encode the credentials
        encoded_credentials = base64.b64encode(f"{consumer_key}:{consumer_secret}".encode()).decode()

        headers = {
            "Authorization": f"Basic {encoded_credentials}",
            "Content-Type": "application/json"
        }

        # Send the request and parse the response
        response = requests.get(url, headers=headers).json()

        # Check for errors and return the access token
        if "access_token" in response:
            return response["access_token"]
        else:
            raise Exception("Failed to get access token: " + response["error_description"])
    except Exception as e:
        raise Exception("Failed to get access token: " + str(e)) 


def generate_stk_password(shortcode: str, passkey: str) -> str:
    timestamp = datetime.now().strftime('%Y%m%d%H%M%S')
    # Create the STK password
    stk_password = base64.b64encode((shortcode + passkey + timestamp).encode('utf-8')).decode('utf-8')
    return stk_password



from setuptools import setup, find_packages

setup(
    name="daraja_client",  # Replace with your package name
    version="0.1.0",    # Initial version
    author="Peter Nyando",
    author_email="peternyando2@gmail.com",
    description=(
        "This python module provides a simple way to integrate the Safaricom MPESA Daraja 2.0 API "
        "into your Python projects. It is designed to handle common operations like generating access "
        "tokens, formatting phone numbers, and sending STK Push requests."
    ),
    long_description=open("README.md").read(),
    long_description_content_type="text/markdown",
    url="https://github.com/anomalous254/daraja_client",  # Replace with your repo URL
    packages=find_packages(),
    classifiers=[
        "Programming Language :: Python :: 3",
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",
    ],
    python_requires='>=3.7',
    install_requires=[
        "certifi==2024.8.30",
        "charset-normalizer==3.4.0",
        "idna==3.10",
        "requests==2.32.3",
        "urllib3==2.2.3",
        "python-decouple==3.8",
    ],
)
